@@ -17,8 +17,8 @@
 #ifndef wasm_ir_properties_h
 #define wasm_ir_properties_h
 
-#include "wasm.h"
 #include "ir/bits.h"
+#include "wasm.h"
 
 namespace wasm {
 
@@ -36,7 +36,7 @@ inline bool emitsBoolean(Expression* curr) {
 inline bool isSymmetric(Binary* binary) {
   switch (binary->op) {
     case AddInt32:
-    case MulInt32: 
+    case MulInt32:
     case AndInt32:
     case OrInt32:
     case XorInt32:
@@ -44,14 +44,16 @@ inline bool isSymmetric(Binary* binary) {
     case NeInt32:
 
     case AddInt64:
-    case MulInt64: 
+    case MulInt64:
     case AndInt64:
     case OrInt64:
     case XorInt64:
     case EqInt64:
-    case NeInt64: return true;
+    case NeInt64:
+      return true;
 
-    default: return false;
+    default:
+      return false;
   }
 }
 
@@ -105,7 +107,8 @@ inline Expression* getAlmostSignExt(Expression* curr) {
           if (auto* inner = outer->left->dynCast<Binary>()) {
             if (inner->op == ShlInt32) {
               if (auto* innerConst = inner->right->dynCast<Const>()) {
-                if (Bits::getEffectiveShifts(outerConst) <= Bits::getEffectiveShifts(innerConst)) {
+                if (Bits::getEffectiveShifts(outerConst) <=
+                    Bits::getEffectiveShifts(innerConst)) {
                   return inner->left;
                 }
               }
@@ -121,7 +124,8 @@ inline Expression* getAlmostSignExt(Expression* curr) {
 // gets the size of the almost sign-extended value, as well as the
 // extra shifts, if any
 inline Index getAlmostSignExtBits(Expression* curr, Index& extraShifts) {
-  extraShifts = Bits::getEffectiveShifts(curr->cast<Binary>()->left->cast<Binary>()->right) -
+  extraShifts = Bits::getEffectiveShifts(
+                  curr->cast<Binary>()->left->cast<Binary>()->right) -
                 Bits::getEffectiveShifts(curr->cast<Binary>()->right);
   return getSignExtBits(curr);
 }
@@ -143,18 +147,19 @@ inline Expression* getZeroExtValue(Expression* curr) {
 
 // gets the size of the sign-extended value
 inline Index getZeroExtBits(Expression* curr) {
-  return Bits::getMaskedBits(curr->cast<Binary>()->right->cast<Const>()->value.geti32());
+  return Bits::getMaskedBits(
+    curr->cast<Binary>()->right->cast<Const>()->value.geti32());
 }
 
-// Returns a falling-through value, that is, it looks through a tee_local
+// Returns a falling-through value, that is, it looks through a local.tee
 // and other operations that receive a value and let it flow through them.
 inline Expression* getFallthrough(Expression* curr) {
   // If the current node is unreachable, there is no value
   // falling through.
-  if (curr->type == unreachable) {
+  if (curr->type == Type::unreachable) {
     return curr;
   }
-  if (auto* set = curr->dynCast<SetLocal>()) {
+  if (auto* set = curr->dynCast<LocalSet>()) {
     if (set->isTee()) {
       return getFallthrough(set->value);
     }
@@ -168,9 +173,9 @@ inline Expression* getFallthrough(Expression* curr) {
   } else if (auto* iff = curr->dynCast<If>()) {
     if (iff->ifFalse) {
       // Perhaps just one of the two actually returns.
-      if (iff->ifTrue->type == unreachable) {
+      if (iff->ifTrue->type == Type::unreachable) {
         return getFallthrough(iff->ifFalse);
-      } else if (iff->ifFalse->type == unreachable) {
+      } else if (iff->ifFalse->type == Type::unreachable) {
         return getFallthrough(iff->ifTrue);
       }
     }
@@ -182,9 +187,12 @@ inline Expression* getFallthrough(Expression* curr) {
   return curr;
 }
 
-} // Properties
+inline bool isConstantExpression(const Expression* curr) {
+  return curr->is<Const>() || curr->is<RefNull>() || curr->is<RefFunc>();
+}
 
-} // wasm
+} // namespace Properties
+
+} // namespace wasm
 
 #endif // wasm_ir_properties_h
-
