@@ -37,15 +37,15 @@
 #include <algorithm>
 #include <memory>
 
-#include <wasm.h>
-#include <wasm-builder.h>
-#include <wasm-traversal.h>
-#include <pass.h>
+#include "ir/flat.h"
 #include <ir/cost.h>
 #include <ir/effects.h>
 #include <ir/equivalent_sets.h>
-#include "ir/flat.h"
 #include <ir/hashed.h>
+#include <pass.h>
+#include <wasm-builder.h>
+#include <wasm-traversal.h>
+#include <wasm.h>
 
 namespace wasm {
 
@@ -60,7 +60,11 @@ struct LocalCSE : public WalkerPass<LinearExecutionWalker<LocalCSE>> {
     Index index; // the local we are assigned to, local.get that to reuse us
     EffectAnalyzer effects;
 
-    UsableInfo(Expression* value, Index index, PassOptions& passOptions) : value(value), index(index), effects(passOptions, value) {}
+    UsableInfo(Expression* value,
+               Index index,
+               PassOptions& passOptions,
+               FeatureSet features)
+      : value(value), index(index), effects(passOptions, features, value) {}
   };
 
   // a list of usables in a linear execution trace
@@ -113,7 +117,7 @@ struct LocalCSE : public WalkerPass<LinearExecutionWalker<LocalCSE>> {
       // the loop above - just the values. So here we must check that
       // sets do not interfere. (Note that due to flattening we
       // have no risk of tees etc.)
-      if (auto* set = curr->dynCast<SetLocal>()) {
+      if (auto* set = curr->dynCast<LocalSet>()) {
         for (auto& sinkable : usables) {
           // Check if the index is the same. Make sure to ignore
           // our own value, which we may have just added!
@@ -135,7 +139,7 @@ struct LocalCSE : public WalkerPass<LinearExecutionWalker<LocalCSE>> {
     // pre operations
     Expression* curr = *currp;
 
-    EffectAnalyzer effects(self->getPassOptions());
+    EffectAnalyzer effects(self->getPassOptions(), self->getModule()->features);
     if (effects.checkPre(curr)) {
       self->checkInvalidations(effects);
     }
@@ -151,7 +155,7 @@ struct LocalCSE : public WalkerPass<LinearExecutionWalker<LocalCSE>> {
 
     // post operations
 
-    EffectAnalyzer effects(self->getPassOptions());
+    EffectAnalyzer effects(self->getPassOptions(), self->getModule()->features);
     if (effects.checkPost(curr)) {
       self->checkInvalidations(effects, curr);
     }
@@ -169,11 +173,14 @@ struct LocalCSE : public WalkerPass<LinearExecutionWalker<LocalCSE>> {
   }
 
   void handle(Expression* curr) {
-    if (auto* set = curr->dynCast<SetLocal>()) {
+    if (auto* set = curr->dynCast<LocalSet>()) {
       // Calculate equivalences
+      auto* func = getFunction();
       equivalences.reset(set->index);
-      if (auto* get = set->value->dynCast<GetLocal>()) {
-        equivalences.add(set->index, get->index);
+      if (auto* get = set->value->dynCast<LocalGet>()) {
+        if (func->getLocalType(set->index) == func->getLocalType(get->index)) {
+          equivalences.add(set->index, get->index);
+        }
       }
       // consider the value
       auto* value = set->value;
@@ -183,14 +190,19 @@ struct LocalCSE : public WalkerPass<LinearExecutionWalker<LocalCSE>> {
         if (iter != usables.end()) {
           // already exists in the table, this is good to reuse
           auto& info = iter->second;
-          set->value = Builder(*getModule()).makeGetLocal(info.index, value->type);
+          Type localType = func->getLocalType(info.index);
+          set->value =
+            Builder(*getModule()).makeLocalGet(info.index, localType);
           anotherPass = true;
         } else {
           // not in table, add this, maybe we can help others later
-          usables.emplace(std::make_pair(hashed, UsableInfo(value, set->index, getPassOptions())));
+          usables.emplace(std::make_pair(
+            hashed,
+            UsableInfo(
+              value, set->index, getPassOptions(), getModule()->features)));
         }
       }
-    } else if (auto* get = curr->dynCast<GetLocal>()) {
+    } else if (auto* get = curr->dynCast<LocalGet>()) {
       if (auto* set = equivalences.getEquivalents(get->index)) {
         // Canonicalize to the lowest index. This lets hashing and comparisons
         // "just work".
@@ -202,13 +214,14 @@ struct LocalCSE : public WalkerPass<LinearExecutionWalker<LocalCSE>> {
   // A relevant value is a non-trivial one, something we may want to reuse
   // and are able to.
   bool isRelevant(Expression* value) {
-    if (value->is<GetLocal>()) {
+    if (value->is<LocalGet>()) {
       return false; // trivial, this is what we optimize to!
     }
-    if (!isConcreteType(value->type)) {
+    if (!value->type.isConcrete()) {
       return false; // don't bother with unreachable etc.
     }
-    if (EffectAnalyzer(getPassOptions(), value).hasSideEffects()) {
+    if (EffectAnalyzer(getPassOptions(), getModule()->features, value)
+          .hasSideEffects()) {
       return false; // we can't combine things with side effects
     }
     auto& options = getPassRunner()->options;
@@ -227,8 +240,6 @@ struct LocalCSE : public WalkerPass<LinearExecutionWalker<LocalCSE>> {
   }
 };
 
-Pass *createLocalCSEPass() {
-  return new LocalCSE();
-}
+Pass* createLocalCSEPass() { return new LocalCSE(); }
 
 } // namespace wasm
