@@ -27,27 +27,22 @@ mod tests {
         // see https://github.com/WebAssembly/binaryen/blob/master/test/example/c-api-hello-world.c
         unsafe {
             let module = BinaryenModuleCreate();
-            let mut params = [BinaryenInt32(), BinaryenInt32()];
+            let mut params = [BinaryenTypeInt32(), BinaryenTypeInt32()];
 
-            let func_type_name = CString::new("iii").unwrap();
-            let iii = BinaryenAddFunctionType(
-                module,
-                func_type_name.as_ptr(),
-                BinaryenInt32(),
-                &mut params[0] as *mut BinaryenType, // TODO: Is this safe?
-                2,
-            );
+            let params = BinaryenTypeCreate(params.as_mut_ptr(), 2);
+            let results = BinaryenTypeInt32();
 
-            let x = BinaryenGetLocal(module, 0, BinaryenInt32());
-            let y = BinaryenGetLocal(module, 1, BinaryenInt32());
+            let x = BinaryenLocalGet(module, 0, BinaryenTypeInt32());
+            let y = BinaryenLocalGet(module, 1, BinaryenTypeInt32());
             let add = BinaryenBinary(module, BinaryenAddInt32(), x, y);
 
             let func_name = CString::new("adder").unwrap();
             let _ = BinaryenAddFunction(
                 module,
                 func_name.as_ptr(),
-                iii,
-                ptr::null::<BinaryenType>() as *mut BinaryenType,
+                params,
+                results,
+                ptr::null_mut(),
                 0,
                 add,
             );
